/* @flow */
'use strict';

var _ = require('highland');
var fs = require('fs');
var path = require('path');
var child = require('child_process');
var spawn = child.spawn;
var exec = child.exec;
var Cmd = require('./ogr-cmd');

function createStream(options) {
  var lastMessage = undefined;
  var readStream = _();
  var cmd = Cmd.create('zip', options);
  var ogr = spawn('ogr2ogr', cmd).on('close', function (c) {
    if (c > 0) readStream.emit('error', new Error('OGR Failed: ' + lastMessage));else {
      var folder = options.path + '/' + options.name;
      if (options.metadata) fs.writeFileSync(folder + '/' + options.name + '.xml', options.metadata);
      createZipStream(folder).on('error', function (e) {
        return readStream.emit('error', e);
      }).pipe(readStream);
    }
  });
  ogr.stderr.on('data', function (data) {
    var msg = data.toString();
    lastMessage = msg;
    readStream.emit('log', { level: 'error', message: msg });
    if (msg.match(/ERROR\s[^6]/)) {
      readStream.emit('error', new Error(msg));
      ogr.kill();
    }
  });
  return readStream;
}

function createZipStream(input) {
  var zipStream = _();
  var cmd = ['zip', '-rj', '"' + input + '.zip"', '"' + input + '"', '--exclude=*.json*', '--exclude=*.vrt'];
  renameFiles(input);
  var zip = exec(cmd.join(' '), function (err, stdout, stderr) {
    if (err) return zipStream.emit('error', new Error('Failed while zipping'));
    fs.createReadStream(input + '.zip').pipe(zipStream);
  });
  process.on('SIGTERM', function () {
    return zip.kill();
  });
  return zipStream;
}

function renameFiles(input) {
  fs.readdirSync(input).forEach(function (file) {
    var oldName = path.join(input, file);
    var newName = oldName.replace(/ogrgeojson/i, path.basename(input));
    fs.renameSync(oldName, newName);
  });
}

module.exports = { createStream: createStream };